import numpy


def rotate_3d(
    verts: numpy.ndarray, x: float, y: float, z: float, dx: float, dy: float, dz: float
) -> numpy.ndarray:
    sb, sh, sa = numpy.sin(numpy.radians([x, y, z]))
    cb, ch, ca = numpy.cos(numpy.radians([x, y, z]))
    trmtx = numpy.array(
        [
            [ch * ca, -ch * sa * cb + sh * sb, ch * sa * sb + sh * cb],
            [sa, ca * cb, -ca * sb],
            [-sh * ca, sh * sa * cb + ch * sb, -sh * sa * sb + ch * cb],
        ]
    )
    origin = numpy.array([dx, dy, dz])
    return numpy.matmul(verts - origin, trmtx) + origin  # type: ignore



import os
import json

from amulet.resource_pack.abc import BaseResourcePack


class JavaResourcePack(BaseResourcePack):
    """A class to hold the bare bones information about the resource pack.
    Holds the pack format, description and if the pack is valid.
    This information can be used in a viewer to display the packs to the user."""

    def __init__(self, resource_pack_path: str):
        super().__init__(resource_pack_path)
        meta_path = os.path.join(resource_pack_path, "pack.mcmeta")
        self._pack_format = 0
        if os.path.isfile(meta_path):
            try:
                with open(meta_path) as f:
                    pack_mcmeta = json.load(f)
            except json.JSONDecodeError:
                pass
            else:
                if "pack" in pack_mcmeta:
                    if "description" in pack_mcmeta["pack"]:
                        self._pack_description = str(pack_mcmeta["pack"]["description"])
                    if "pack_format" in pack_mcmeta["pack"]:
                        self._pack_format = pack_mcmeta["pack"]["pack_format"]
                        self._valid_pack = True

        pack_icon_path = os.path.join(resource_pack_path, "pack.png")
        if os.path.isfile(pack_icon_path):
            self._pack_icon = pack_icon_path

    @staticmethod
    def is_valid(pack_path: str) -> bool:
        return os.path.isfile(os.path.join(pack_path, "pack.mcmeta"))

    def __repr__(self) -> str:
        return f"JavaResourcePack({self._root_dir})"

    @property
    def pack_format(self) -> int:
        """int - pack format number"""
        return self._pack_format



import os
from packaging.version import Version

AMULET_COMPILER_TARGET_REQUIREMENT = "==2.0"
AMULET_COMPILER_VERSION_REQUIREMENT = "==3.0.0"

PYBIND11_REQUIREMENT = "==2.13.6"
AMULET_PYBIND11_EXTENSIONS_REQUIREMENT = "~=1.1.0.0a0"
AMULET_IO_REQUIREMENT = "~=1.0"
AMULET_UTILS_REQUIREMENT = "~=1.1.0.0a6"
AMULET_ZLIB_REQUIREMENT = "~=1.0.0.0a7"
AMULET_NBT_REQUIREMENT = "~=5.0.0.0a6"
AMULET_CORE_REQUIREMENT = "~=2.0.3.0a1"
NUMPY_REQUIREMENT = "~=2.0"
PIL_REQUIREMENT = "~=11.0"

if os.environ.get("AMULET_PYBIND11_EXTENSIONS_REQUIREMENT", None):
    AMULET_PYBIND11_EXTENSIONS_REQUIREMENT = f"{AMULET_PYBIND11_EXTENSIONS_REQUIREMENT},{os.environ['AMULET_PYBIND11_EXTENSIONS_REQUIREMENT']}"

if os.environ.get("AMULET_IO_REQUIREMENT", None):
    AMULET_IO_REQUIREMENT = (
        f"{AMULET_IO_REQUIREMENT},{os.environ['AMULET_IO_REQUIREMENT']}"
    )

if os.environ.get("AMULET_UTILS_REQUIREMENT", None):
    AMULET_UTILS_REQUIREMENT = (
        f"{AMULET_UTILS_REQUIREMENT},{os.environ['AMULET_UTILS_REQUIREMENT']}"
    )

if os.environ.get("AMULET_ZLIB_REQUIREMENT", None):
    AMULET_ZLIB_REQUIREMENT = (
        f"{AMULET_ZLIB_REQUIREMENT},{os.environ['AMULET_ZLIB_REQUIREMENT']}"
    )

if os.environ.get("AMULET_NBT_REQUIREMENT", None):
    AMULET_NBT_REQUIREMENT = (
        f"{AMULET_NBT_REQUIREMENT},{os.environ['AMULET_NBT_REQUIREMENT']}"
    )

if os.environ.get("AMULET_CORE_REQUIREMENT", None):
    AMULET_CORE_REQUIREMENT = (
        f"{AMULET_CORE_REQUIREMENT},{os.environ['AMULET_CORE_REQUIREMENT']}"
    )


def get_specifier_set(version_str: str) -> str:
    """
    version_str: The PEP 440 version number of the library.
    """
    version = Version(version_str)
    if version.epoch != 0 or version.is_devrelease or version.is_postrelease:
        raise RuntimeError(f"Unsupported version format. {version_str}")

    return f"~={version.major}.{version.minor}.{version.micro}.0{''.join(map(str, version.pre or ()))}"


if os.environ.get("AMULET_FREEZE_COMPILER", None):
    import get_compiler

    AMULET_COMPILER_VERSION_REQUIREMENT = get_compiler.main()

    try:
        import amulet.pybind11_extensions
    except ImportError:
        pass
    else:
        AMULET_PYBIND11_EXTENSIONS_REQUIREMENT = get_specifier_set(
            amulet.pybind11_extensions.__version__
        )

    try:
        import amulet.io
    except ImportError:
        pass
    else:
        AMULET_IO_REQUIREMENT = get_specifier_set(amulet.io.__version__)

    try:
        import amulet.utils
    except ImportError:
        pass
    else:
        AMULET_UTILS_REQUIREMENT = get_specifier_set(amulet.utils.__version__)

    try:
        import amulet.zlib
    except ImportError:
        pass
    else:
        AMULET_ZLIB_REQUIREMENT = get_specifier_set(amulet.zlib.__version__)

    try:
        import amulet.nbt
    except ImportError:
        pass
    else:
        AMULET_NBT_REQUIREMENT = get_specifier_set(amulet.nbt.__version__)

    try:
        import amulet.core
    except ImportError:
        pass
    else:
        AMULET_CORE_REQUIREMENT = get_specifier_set(amulet.core.__version__)


def get_build_dependencies() -> list:
    return [
        f"amulet-compiler-version{AMULET_COMPILER_VERSION_REQUIREMENT}",
        f"pybind11{PYBIND11_REQUIREMENT}",
        f"amulet-pybind11-extensions{AMULET_PYBIND11_EXTENSIONS_REQUIREMENT}",
        f"amulet-io{AMULET_IO_REQUIREMENT}",
        f"amulet-utils{AMULET_UTILS_REQUIREMENT}",
        f"amulet-zlib{AMULET_ZLIB_REQUIREMENT}",
        f"amulet-nbt{AMULET_NBT_REQUIREMENT}",
        f"amulet-core{AMULET_CORE_REQUIREMENT}",
    ] * (not os.environ.get("AMULET_SKIP_COMPILE", None))


def get_runtime_dependencies() -> list[str]:
    return [
        f"amulet-compiler-target{AMULET_COMPILER_TARGET_REQUIREMENT}",
        f"amulet-compiler-version{AMULET_COMPILER_VERSION_REQUIREMENT}",
        f"pybind11{PYBIND11_REQUIREMENT}",
        f"amulet-pybind11-extensions{AMULET_PYBIND11_EXTENSIONS_REQUIREMENT}",
        f"amulet-io{AMULET_IO_REQUIREMENT}",
        f"amulet-utils{AMULET_UTILS_REQUIREMENT}",
        f"amulet-zlib{AMULET_ZLIB_REQUIREMENT}",
        f"amulet-nbt{AMULET_NBT_REQUIREMENT}",
        f"amulet-core{AMULET_CORE_REQUIREMENT}",
        f"numpy{NUMPY_REQUIREMENT}",
        f"pillow{PIL_REQUIREMENT}",
    ]



from amulet.resource_pack.image import missing_pack_icon_path


class BaseResourcePack:
    """The base class that all resource packs must inherit from. Defines the base api."""

    def __init__(self, root_dir: str):
        self._valid_pack = False
        self._root_dir = root_dir
        self._pack_description = ""
        self._pack_icon = missing_pack_icon_path

    def __repr__(self) -> str:
        raise NotImplementedError

    @staticmethod
    def is_valid(pack_path: str) -> bool:
        raise NotImplementedError

    @property
    def valid_pack(self) -> bool:
        """bool - does the pack meet the minimum requirements to be a resource pack"""
        return self._valid_pack

    @property
    def root_dir(self) -> str:
        """str - the root directory of the pack"""
        return self._root_dir

    @property
    def pack_description(self) -> str:
        """str - the description as described in the pack"""
        return self._pack_description

    @property
    def pack_icon(self) -> str:
        """str - path to the pack icon"""
        return self._pack_icon



from __future__ import annotations

from amulet.resource_pack.java.resource_pack import JavaResourcePack
from amulet.resource_pack.java.resource_pack_manager import JavaResourcePackManager

from . import resource_pack, resource_pack_manager

__all__ = [
    "JavaResourcePack",
    "JavaResourcePackManager",
    "resource_pack",
    "resource_pack_manager",
]



from typing import Iterable, Union

from .abc import BaseResourcePack, BaseResourcePackManager
from .java import JavaResourcePack, JavaResourcePackManager
from .unknown_resource_pack import UnknownResourcePack


def load_resource_pack(resource_pack_path: str) -> BaseResourcePack:
    if JavaResourcePack.is_valid(resource_pack_path):
        return JavaResourcePack(resource_pack_path)
    # elif BedrockResourcePack.is_valid(resource_pack_path):
    #     return BedrockResourcePack(resource_pack_path)
    else:
        return UnknownResourcePack(resource_pack_path)


def load_resource_pack_manager(
    resource_packs: Iterable[Union[str, BaseResourcePack]], load: bool = True
) -> BaseResourcePackManager:
    resource_packs_out: list[BaseResourcePack] = []
    for resource_pack in resource_packs:
        if isinstance(resource_pack, str):
            resource_pack = load_resource_pack(resource_pack)
        if (
            not isinstance(resource_pack, UnknownResourcePack)
            and resource_pack.valid_pack
        ):
            if resource_packs_out:
                if isinstance(resource_pack, resource_packs_out[0].__class__):
                    resource_packs_out.append(resource_pack)
            else:
                resource_packs_out.append(resource_pack)

    resource_packs = resource_packs_out
    if resource_packs:
        if isinstance(resource_packs[0], JavaResourcePack):
            return JavaResourcePackManager(
                [pack for pack in resource_packs if isinstance(pack, JavaResourcePack)],
                load,
            )
        # elif isinstance(resource_packs[0], BedrockResourcePack):
        #     return BedrockResourcePackManager(
        #         [
        #             pack
        #             for pack in resource_packs
        #             if isinstance(pack, BedrockResourcePack)
        #         ],
        #         load,
        #     )

    raise NotImplementedError
    # return UnknownResourcePackManager()
